@@ -19,7 +19,7 @@ import threading
 import gi
 gi.require_version('Gtk', '3.0')
 gi.require_version('XApp', '1.0')
-from gi.repository import Gtk, Gdk, GdkPixbuf, GObject, Pango, XApp
+from gi.repository import Gtk, Gdk, GdkPixbuf, GLib, GObject, Pango, XApp
 
 from aptsources.sourceslist import SourcesList
 from io import BytesIO
@@ -208,7 +208,7 @@ def add_key_remote(key):
     return True
 
 def repo_malformed(line):
-    r = re.compile(r'(?:deb|deb-src)\s+\w+:/\S+?/?\s+\S+')
+    r = re.compile(r'(?:deb|deb-src)\s+(?:\[[^\]]+\]\s+)?\w+:/\S+?/?\s+\S+')
     match_line = r.match(line)
     if not match_line:
         return True
@@ -344,7 +344,7 @@ class Key():
         subprocess.call(["apt-key", "del", self.pub])
 
     def get_name(self):
-        return "%s\n<small>    %s</small>" % (GObject.markup_escape_text(self.uid), GObject.markup_escape_text(self.pub))
+        return "%s\n<small>    %s</small>" % (GLib.markup_escape_text(self.uid), GLib.markup_escape_text(self.pub))
 
 class Mirror():
     def __init__(self, country_code, url, name):
@@ -436,7 +436,7 @@ class Repository():
                                     name = subparts[-2].capitalize()
                                 else:
                                     name = subparts[-3].capitalize()
-                            name = name.replace("pearllinux", "Pearl Linux")
+                            name = name.replace("Linuxpearl", "Pearl Linux")
                             name = name.replace("01", "Intel")
                             name = name.replace("Steampowered", "Steam")
                         break
@@ -453,7 +453,7 @@ class ComponentSwitchBox(Gtk.Box):
         self.component = component
         self.window_object = window
         Gtk.Box.__init__(self)
-        label = Gtk.Label(self.component.description)
+        label = Gtk.Label(label=self.component.description)
         self.pack_start(label, False, False, 0)
         self.switch = Gtk.Switch()
         self.pack_end(self.switch, False, False, 0)
@@ -522,7 +522,7 @@ class MirrorSelectionDialog(object):
         col.set_sort_column_id(MirrorSelectionDialog.MIRROR_TOOLTIP_COLUMN)
 
         r = Gtk.CellRendererText()
-        col = Gtk.TreeViewColumn(_("URL"), r, text = MirrorSelectionDialog.MIRROR_NAME_COLUMN)
+        col = Gtk.TreeViewColumn(_("Mirror"), r, text = MirrorSelectionDialog.MIRROR_NAME_COLUMN)
         self._treeview.append_column(col)
         col.set_sort_column_id(MirrorSelectionDialog.MIRROR_NAME_COLUMN)
 
@@ -566,7 +566,7 @@ class MirrorSelectionDialog(object):
                 flag = FLAG_PATH % '_generic'
             tooltip = country_name
             if mirror.name != mirror.url:
-                tooltip = "%s: %s" % (country_name, mirror.name)
+                tooltip = "%s: %s" % (country_name, mirror.url)
             pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(flag, -1, FLAG_SIZE * self._application.scale)
             self._mirrors_model.append((
                 mirror,
@@ -598,23 +598,31 @@ class MirrorSelectionDialog(object):
         except:
             return None
 
-    def check_mirror_up_to_date(self, url):
-        if (self.default_mirror_age is None or self.default_mirror_age < 2):
-            # If the default server was updated recently, the age is irrelevant (it would measure the time between now and the last update)
-            return True
+    def check_mirror_up_to_date(self, url, max_age):
         mirror_timestamp = self.get_url_last_modified(url)
         if mirror_timestamp is None:
             print ("Error: Can't find the age of %s !!" % url)
             return False
         mirror_date = datetime.datetime.fromtimestamp(mirror_timestamp)
         mirror_age = (self.default_mirror_date - mirror_date).days
-        if (mirror_age > 2):
+        # print("age: %d, max: %d - %s"%(mirror_age, max_age, url))
+        if (mirror_age > max_age):
             print ("Error: %s is out of date by %d days!" % (url, mirror_age))
             return False
         else:
             # Age is fine :)
             return True
 
+    def check_pearl_mirror_up_to_date(self, url):
+        if (self.default_mirror_age is None or self.default_mirror_age < 2):
+            # print("Skipping Pearl mirror check, < 2 days old: %s" % url)
+            # If the default server was updated recently, the age is irrelevant (it would measure the time between now and the last update)
+            return True
+        return self.check_mirror_up_to_date(url, 2)
+
+    def check_base_mirror_up_to_date(self, url):
+        return self.check_mirror_up_to_date(url, 14)
+
     @run_async
     def _all_speed_tests(self):
         model_iters = [] # Don't iterate through iters directly.. we're modifying their orders..
@@ -657,11 +665,16 @@ class MirrorSelectionDialog(object):
     def _speed_test(self, iter, url):
         download_speed = 0
         try:
+            up_to_date = False
+
             if self.is_base:
                 test_url = "%s/dists/%s/main/binary-amd64/Packages.gz" % (url, self.codename)
+                up_to_date = self.check_base_mirror_up_to_date("%s/ls-lR.gz" % url)
             else:
                 test_url = "%s/dists/%s/main/Contents-amd64.gz" % (url, self.codename)
-            if (self.is_base or self.check_mirror_up_to_date("%s/db/version" % url)):
+                up_to_date = self.check_pearl_mirror_up_to_date("%s/db/version" % url)
+
+            if up_to_date:
                 c = pycurl.Curl()
                 buff = BytesIO()
                 c.setopt(pycurl.URL, test_url)
@@ -773,7 +786,12 @@ class MirrorSelectionDialog(object):
         # Try to find the age of the Pearl archive
         self.default_mirror_age = None
         self.default_mirror_date = None
-        mirror_timestamp = self.get_url_last_modified("%s/db/version" % self.default_mirror)
+
+        if self.is_base:
+            mirror_timestamp = self.get_url_last_modified("%s/ls-lR.gz" % self.default_mirror)
+        else:
+            mirror_timestamp = self.get_url_last_modified("%s/db/version" % self.default_mirror)
+
         if mirror_timestamp is not None:
             self.default_mirror_date = datetime.datetime.fromtimestamp(mirror_timestamp)
             now = datetime.datetime.now()
@@ -802,8 +820,6 @@ class Application(object):
         self._interface_loaded = False
         self._currently_applying_sources = False
 
-        self.infobar_visible = False
-
         self.lsb_codename = subprocess.getoutput("lsb_release -sc")
 
         glade_file = "/usr/lib/pearllinux/pearlSources/pearlsources.glade"
@@ -812,6 +828,8 @@ class Application(object):
         self.builder.set_translation_domain("pearlsources")
         self.builder.add_from_file(glade_file)
         self._main_window = self.builder.get_object("main_window")
+        self._info_box = self.builder.get_object("box_infobar")
+        self._info_revealer = self.builder.get_object("info_revealer")
 
         self._main_window.set_title(_("Software Sources"))
 
@@ -1061,7 +1079,7 @@ class Application(object):
                             mirror = Mirror(country_code, url, name)
                             mirror_list.append(mirror)
         if path.endswith("Debian.mirrors"):
-            mirror = Mirror("WD", "http://deb.debian.org/debian", "http://deb.debian.org/debian/")
+            mirror = Mirror("WD", "https://deb.debian.org/debian", "https://deb.debian.org/debian/")
             mirror_list.append(mirror)
         return mirror_list
 
@@ -1689,27 +1707,28 @@ class Application(object):
 
 
     def enable_reload_button(self):
-        if not self.infobar_visible:
-            self.infobar_visible = True
-            infobar = Gtk.InfoBar()
-            infobar.set_message_type(Gtk.MessageType.INFO)
-            box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=6)
-            image = Gtk.Image.new_from_icon_name("dialog-information-symbolic", Gtk.IconSize.LARGE_TOOLBAR)
-            box.pack_start(image, False, False, 0)
-            info_label = Gtk.Label()
-            infobar_message = "<b>%s</b>\n%s" % (_("Your configuration changed"), _("Click OK to update your APT cache"))
-            info_label.set_markup(infobar_message)
-            box.pack_start(info_label, False, False, 0)
-            infobar.get_content_area().pack_start(box, False, False, 0)
-            infobar.add_button(_("OK"), Gtk.ResponseType.OK)
-            infobar.add_button(_("Cancel"), Gtk.ResponseType.CANCEL)
-            infobar.connect("response", self._on_infobar_response)
-            self.builder.get_object("box_infobar").pack_start(infobar, True, True, 5)
-            infobar.show_all()
+        if not self._info_revealer.get_reveal_child():
+            if self._info_box.get_children() == []:
+                infobar = Gtk.InfoBar()
+                infobar.set_message_type(Gtk.MessageType.INFO)
+                box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=6)
+                image = Gtk.Image.new_from_icon_name("dialog-information-symbolic", Gtk.IconSize.LARGE_TOOLBAR)
+                box.pack_start(image, False, False, 0)
+                info_label = Gtk.Label()
+                infobar_message = "<b>%s</b>\n%s" % (_("Your configuration changed"), _("Click OK to update your APT cache"))
+                info_label.set_markup(infobar_message)
+                box.pack_start(info_label, False, False, 0)
+                infobar.get_content_area().pack_start(box, False, False, 0)
+                infobar.add_button(_("OK"), Gtk.ResponseType.OK)
+                infobar.add_button(_("Cancel"), Gtk.ResponseType.CANCEL)
+                infobar.connect("response", self._on_infobar_response)
+                infobar.show_all()
+                self._info_box.pack_start(infobar, True, True, 0)
+
+            self._info_revealer.set_reveal_child(True)
 
     def _on_infobar_response(self, infobar, response_id):
-        infobar.destroy()
-        self.infobar_visible = False
+        self._info_revealer.set_reveal_child(False)
 
         if response_id == Gtk.ResponseType.OK:
             self.apt.update_cache()
